// BugTrackerView.cpp : implementation of the CBugTrackerView class
//

#include "stdafx.h"
#include "BugTracker.h"

#include "BugTrackerDoc.h"
#include "BugTrackerView.h"
#include "AddEditBugDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

char *arrStatus[] = { "Outstanding", "Fixed" };

/////////////////////////////////////////////////////////////////////////////
// CBugTrackerView

IMPLEMENT_DYNCREATE(CBugTrackerView, CView)

BEGIN_MESSAGE_MAP(CBugTrackerView, CView)
	//{{AFX_MSG_MAP(CBugTrackerView)
	ON_WM_CREATE()
	ON_WM_SIZE()
	ON_WM_TIMER()
	ON_NOTIFY(NM_DBLCLK, ID_LIST_CONTROL, OnDblclkListResult)
	ON_COMMAND(ID_NEW_BUG, OnNewBug)
	ON_COMMAND(ID_REFRESH_BUGS, OnRefreshBugs)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBugTrackerView construction/destruction

CBugTrackerView::CBugTrackerView()
{
	// TODO: add construction code here

}

CBugTrackerView::~CBugTrackerView()
{
}

BOOL CBugTrackerView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CBugTrackerView drawing

void CBugTrackerView::OnDraw(CDC* pDC)
{
	CBugTrackerDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	m_ListCtrl.Invalidate();
}

/////////////////////////////////////////////////////////////////////////////
// CBugTrackerView diagnostics

#ifdef _DEBUG
void CBugTrackerView::AssertValid() const
{
	CView::AssertValid();
}

void CBugTrackerView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CBugTrackerDoc* CBugTrackerView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CBugTrackerDoc)));
	return (CBugTrackerDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CBugTrackerView message handlers

int CBugTrackerView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	CRect rect;

	//create list control
	if (!m_ListCtrl.Create(WS_VISIBLE|WS_CHILD|LVS_REPORT, rect, this, ID_LIST_CONTROL))
		return -1;

	//set extended list control styles
	ListView_SetExtendedListViewStyle(m_ListCtrl.GetSafeHwnd(),
		LVS_EX_FULLROWSELECT | 
		LVS_EX_HEADERDRAGDROP 
		);

	//insert list control columns
	m_ListCtrl.InsertColumn(0, "ID", LVCFMT_LEFT, 100);
	m_ListCtrl.InsertColumn(1, "Date", LVCFMT_LEFT, 100);
	m_ListCtrl.InsertColumn(2, "Reported By", LVCFMT_LEFT, 100);
	m_ListCtrl.InsertColumn(3, "Status", LVCFMT_LEFT, 100);
	m_ListCtrl.InsertColumn(4, "Description", LVCFMT_LEFT, 100);
	m_ListCtrl.InsertColumn(5, "Notes", LVCFMT_LEFT, 100);
	m_ListCtrl.InsertColumn(6, "Priority", LVCFMT_LEFT, 100);
	
	SetTimer(REFRESH_TIMER, 60000, NULL);

	return 0;
}

void CBugTrackerView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	RECT rect;
	GetClientRect(&rect);
	m_ListCtrl.MoveWindow(&rect);	
}

void CBugTrackerView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	//empty list control
	m_ListCtrl.DeleteAllItems();

	//open local bug DB
	CBugsSet bugs;
	bugs.Open(dbOpenTable);
	if (bugs.IsEOF() && bugs.IsBOF())
		return;

	int i=0;
	CString strValue;

	//fill list control with all bugs in local DB
	bugs.MoveFirst();
	while (!bugs.IsEOF())
	{
		LV_ITEM item;
		item.iItem = i++;
		item.mask = LVIF_TEXT;
		item.iSubItem = 0;
		item.cchTextMax = 50;

		strValue.Format("%d", bugs.m_ID);
		item.pszText = strValue.GetBuffer(50);

		// insert item in list ctrl & set column text info
		m_ListCtrl.InsertItem(&item);
		m_ListCtrl.SetItemText(item.iItem, 1, bugs.m_Date.GetBuffer(50));
		m_ListCtrl.SetItemText(item.iItem, 2, bugs.m_ReportedBy.GetBuffer(50));
		m_ListCtrl.SetItemText(item.iItem, 3, arrStatus[bugs.m_Status]);
		m_ListCtrl.SetItemText(item.iItem, 4, bugs.m_Description.GetBuffer(0));
		m_ListCtrl.SetItemText(item.iItem, 5, bugs.m_Notes.GetBuffer(0));
		strValue.Format("%d", bugs.m_Priority);
		m_ListCtrl.SetItemText(item.iItem, 6, strValue.GetBuffer(0));
		m_ListCtrl.EnsureVisible( item.iItem, TRUE );

		//move to next record and continue
		bugs.MoveNext();
	}	
	bugs.Close();
}

void CBugTrackerView::OnDblclkListResult(NMHDR* pNMHDR, LRESULT* pResult) 
{
	// get selected item
	UINT index = m_ListCtrl.GetNextItem(-1 , LVNI_SELECTED);
	if (index == -1)
		return;	
	
	CAddEditBugDlg dlg;
	//initialize add/edit dialog
	dlg.m_nID = atol(m_ListCtrl.GetItemText(index, 0));
	dlg.m_strDate = m_ListCtrl.GetItemText(index, 1);
	dlg.m_strReportedBy = m_ListCtrl.GetItemText(index, 2);
	dlg.m_strStatus = m_ListCtrl.GetItemText(index, 3);
	dlg.m_strDesc = m_ListCtrl.GetItemText(index, 4);
	dlg.m_strNotes = m_ListCtrl.GetItemText(index, 5);
	dlg.m_nPriority = atol(m_ListCtrl.GetItemText(index, 6));

	if (IDCANCEL == dlg.DoModal())
		return;

	//format the request data
	CString strData;
	strData.Format("id=%d&date=%s&reportedby=%s&status=%d&description=%s&notes=%s&priority=%d",
		dlg.m_nID, dlg.m_strDate, dlg.m_strReportedBy, dlg.m_lStatus, dlg.m_strDesc, dlg.m_strNotes, dlg.m_nPriority);
	
	//send the addeditbug request
	UINT nServerType = AfxGetApp()->GetProfileInt(REG_KEY_SETTINGS, "type", 0);
	
	if (TYPE_ASP == nServerType)
		SendRequest("addeditbug.asp", strData);
	else
		SendRequest("bugs.dll?addeditbug", strData);

	*pResult = 0;
}

void CBugTrackerView::OnNewBug() 
{
	CAddEditBugDlg dlg;
	if (IDCANCEL == dlg.DoModal())
		return;

	CString strData;
	//format the request data
	strData.Format("id=0&date=%s&reportedby=%s&status=%d&description=%s&notes=%s&priority=%d",
		dlg.m_strDate, dlg.m_strReportedBy, dlg.m_lStatus, dlg.m_strDesc, dlg.m_strNotes, dlg.m_nPriority);

	//send the addeditbug request
	UINT nServerType = AfxGetApp()->GetProfileInt(REG_KEY_SETTINGS, "type", 0);
	
	if (TYPE_ASP == nServerType)
		SendRequest("addeditbug.asp", strData);
	else
		SendRequest("bugs.dll?addeditbug", strData);
}

void CBugTrackerView::OnRefreshBugs() 
{
	CString strData;

	//send the addeditbug request
	UINT nServerType = AfxGetApp()->GetProfileInt(REG_KEY_SETTINGS, "type", 0);
	
	if (TYPE_ASP == nServerType)
		SendRequest("refreshbugs.asp", strData);
	else
		SendRequest("bugs.dll?refresh", strData);
}

void CBugTrackerView::SendRequest(LPCTSTR lpszObject, CString& strData)
{
	DWORD dwStatus, dwRead;
	CString strResponse;
	CWinApp* pApp = AfxGetApp();

	//get server, port, and path information saved in registry
	CString strServer = pApp->GetProfileString(REG_KEY_SETTINGS, "server", "aarons.axtech.com");
	UINT nPort = pApp->GetProfileInt(REG_KEY_SETTINGS, "port", 80);
	CString strPath = pApp->GetProfileString(REG_KEY_SETTINGS, "path", "/scripts/");

	//add object passed in lpszObject
	strPath += lpszObject;
	strPath += "?";
	strPath += strData;

	TRY
	{
		//get app Internet session and get HTTP connection
		CInternetSession& session = GetDocument()->m_Session;
		CHttpConnection* pConnection = session.GetHttpConnection(strServer, (USHORT)nPort, NULL, NULL);
		if (pConnection)
		{
			//open the HTTP request
			CHttpFile* pFile = pConnection->OpenRequest("GET", strPath);
			pFile->SendRequest(NULL, 0, NULL, 0); //strData.GetBuffer(0), strData.GetLength());

			//check the HTTP status code
			pFile->QueryInfoStatusCode(dwStatus);
			if (dwStatus == HTTP_STATUS_OK)
			{
				//read the response
				char szBuffer[BUFFLEN];
				while(dwRead=pFile->Read(szBuffer, BUFFLEN))
				{
					szBuffer[dwRead]=0;
					strResponse += szBuffer;
				}
				ParseBugs(strResponse.GetBuffer(0));
			}
			//close HTTP file and connection
			pFile->Close();
			pConnection->Close();

			//delete file and connection objects
			if (pFile) 
				delete pFile;
			if (pConnection) 
				delete pConnection;
		}
	}
	CATCH(CInternetException,e)
	{
		e->ReportError();
	}
	END_CATCH
}

void CBugTrackerView::ParseBugs(char *lpszResponse)
{
	CString strBugTag, strBug;
	UINT i=0;

	for (;;)
	{
		strBugTag.Format("Bug%d", i++);
		strBug = GetNameValue(strBugTag.GetBuffer(8), lpszResponse);
		if (strBug.IsEmpty())
			break;
		ParseBug(strBug.GetBuffer(0));
	}
	GetDocument()->UpdateAllViews(NULL);
}

//returns the string associated with 'pName' if one is found, 
//otherwise returns an empty CString
CString CBugTrackerView::GetNameValue(char *pName, char *pResponse)
{
	CString strValue;
	char *p1,*p2;

	p1 = pResponse;

	while (p1 = strstr(p1,pName))
	{
		if ( (p1 == pResponse) || ( *(p1-1) == '>' ) ) 
		{
			p1 += strlen(pName)+1;
			if (p2 = strstr(p1,EOR))
			{
				char cSave = *p2;
				*p2 = NULL;
				strValue = p1;
				*p2 = cSave;
			}
			break;
		}
		//otherwise move to the next line and search again, skip the fake
		else p1+=strlen(pName)+1;	
	}
	return strValue;
}

void CBugTrackerView::ParseBug(char* lpszBug)
{
	char *p1, *p2, *pval;
	CString strDesc, strDate, strRepBy, strNotes;
	long lID, lStatus, lPriority;

	// parse out information
	lID = atol(strtok(lpszBug, SEP));
	strDate = strtok(NULL, SEP);
	strRepBy = strtok(NULL, SEP);
	lStatus = atol(strtok(NULL, SEP));
    strDesc = strtok(NULL, SEP);
	strNotes = strtok(NULL, SEP);
	lPriority = atol(strtok(NULL, SEP));

    // open local database
	CBugsSet bugs;
	bugs.Open(dbOpenTable);
	
	//either edit or add new record
	if (bugs.FindBugByID(lID))
		bugs.Edit();
	else
		bugs.AddNew();

	//fill in recordset members
	bugs.m_ID = lID;
	bugs.m_Date = strDate;
	bugs.m_ReportedBy = strRepBy;
	bugs.m_Status = lStatus;
	bugs.m_Description = strDesc;
	bugs.m_Notes = strNotes;
	bugs.m_Priority = lPriority;
	bugs.Update();
	bugs.Close();
}

//automatically synchs w/bug DB on server
void CBugTrackerView::OnTimer(UINT nIDEvent) 
{
	if (nIDEvent == REFRESH_TIMER)
	{
		OnRefreshBugs();
	}	
	CView::OnTimer(nIDEvent);
}

