// Copyright (c) 1998, Microsoft Systems Journal
// Author: Aaron Skonnard
//
// QuoteProviders.cpp : Implementation of DLL Exports.
//

// Note: Proxy/Stub Information
//		To build a separate proxy/stub DLL, 
//		run nmake -f QuoteProvidersps.mk in the project directory.

#include "stdafx.h"
#include "resource.h"
#include "CategoryGuid.h"
#include "initguid.h"
#include "QuoteProviders.h"

#include "QuoteProviders_i.c"
#include "comcat.h"
#include "ProviderYahoo.h"
#include "ProviderFastQuote.h"
#include "ProviderDatek.h"

HRESULT CreateComponentCategory(CATID catid, WCHAR* catDescription);
HRESULT RegisterCLSIDInCategory(REFCLSID clsid, CATID catid);
HRESULT UnRegisterCLSIDInCategory(REFCLSID clsid, CATID catid);   

CRITICAL_SECTION g_CS;

CComModule _Module;

BEGIN_OBJECT_MAP(ObjectMap)
	OBJECT_ENTRY(CLSID_ProviderYahoo, CProviderYahoo)
	OBJECT_ENTRY(CLSID_ProviderFastQuote, CProviderFastQuote)
	OBJECT_ENTRY(CLSID_ProviderDatek, CProviderDatek)
END_OBJECT_MAP()

class CQuoteProvidersApp : public CWinApp
{
public:
	virtual BOOL InitInstance();
	virtual int ExitInstance();
};

CQuoteProvidersApp theApp;

BOOL CQuoteProvidersApp::InitInstance()
{
	_Module.Init(ObjectMap, m_hInstance);
	return CWinApp::InitInstance();
}

int CQuoteProvidersApp::ExitInstance()
{
	_Module.Term();
	return CWinApp::ExitInstance();
}

/////////////////////////////////////////////////////////////////////////////
// Used to determine whether the DLL can be unloaded by OLE

STDAPI DllCanUnloadNow(void)
{
	AFX_MANAGE_STATE(AfxGetStaticModuleState());
	return (AfxDllCanUnloadNow()==S_OK && _Module.GetLockCount()==0) ? S_OK : S_FALSE;
}

/////////////////////////////////////////////////////////////////////////////
// Returns a class factory to create an object of the requested type

STDAPI DllGetClassObject(REFCLSID rclsid, REFIID riid, LPVOID* ppv)
{
	return _Module.GetClassObject(rclsid, riid, ppv);
}

/////////////////////////////////////////////////////////////////////////////
// DllRegisterServer - Adds entries to the system registry

STDAPI DllRegisterServer(void)
{
	HRESULT hr;
	// registers object, typelib and all interfaces in typelib
	hr = _Module.RegisterServer(TRUE);
	if (FAILED(hr))
		return hr;
    hr = CreateComponentCategory(CATID_QuoteProviders, L"Stock Watcher Quote Providers");    
	if (FAILED(hr))
        return hr;
    hr = RegisterCLSIDInCategory(CLSID_ProviderYahoo, CATID_QuoteProviders);
    if (FAILED(hr))        
		return hr;
    hr = RegisterCLSIDInCategory(CLSID_ProviderFastQuote, CATID_QuoteProviders);
    if (FAILED(hr))        
		return hr;
    hr = RegisterCLSIDInCategory(CLSID_ProviderDatek, CATID_QuoteProviders);
    if (FAILED(hr))        
		return hr;
	return S_OK;
}

/////////////////////////////////////////////////////////////////////////////
// DllUnregisterServer - Removes entries from the system registry

STDAPI DllUnregisterServer(void)
{
	HRESULT hr;
	_Module.UnregisterServer();
    hr=UnRegisterCLSIDInCategory(CLSID_ProviderYahoo, CATID_QuoteProviders);
    if (FAILED(hr))        
		return hr;
    hr=UnRegisterCLSIDInCategory(CLSID_ProviderFastQuote, CATID_QuoteProviders);
    if (FAILED(hr))        
		return hr;
    hr=UnRegisterCLSIDInCategory(CLSID_ProviderDatek, CATID_QuoteProviders);
    if (FAILED(hr))        
		return hr;
	return S_OK;
}


HRESULT CreateComponentCategory(CATID catid, WCHAR* catDescription)
{

    ICatRegister* pcr = NULL ;
    HRESULT hr = S_OK ;

    hr = CoCreateInstance(CLSID_StdComponentCategoriesMgr, 
            NULL, CLSCTX_INPROC_SERVER, IID_ICatRegister, (void**)&pcr);
    if (FAILED(hr))
        return hr;

    // Make sure the HKCR\Component Categories\{..catid...}
    // key is registered.
    CATEGORYINFO catinfo;
    catinfo.catid = catid;
    catinfo.lcid = 0x0409 ; // english

    // Make sure the provided description is not too long.
    // Only copy the first 127 characters if it is.
    int len = wcslen(catDescription);
    if (len>127)
        len = 127;
    wcsncpy(catinfo.szDescription, catDescription, len);
    // Make sure the description is null terminated.
    catinfo.szDescription[len] = _T('\0');

    hr = pcr->RegisterCategories(1, &catinfo);
        pcr->Release();

    return hr;
}

HRESULT RegisterCLSIDInCategory(REFCLSID clsid, CATID catid)
{
// Register your component categories information.
    ICatRegister* pcr = NULL ;
    HRESULT hr = S_OK ;
    hr = CoCreateInstance(CLSID_StdComponentCategoriesMgr, 
                NULL, CLSCTX_INPROC_SERVER, IID_ICatRegister, (void**)&pcr);
    if (SUCCEEDED(hr))
    {
       // Register this category as being "implemented" by the class.
       CATID rgcatid[1] ;
       rgcatid[0] = catid;
       hr = pcr->RegisterClassImplCategories(clsid, 1, rgcatid);
    }

    if (pcr != NULL)
        pcr->Release();

    return hr;
}

HRESULT UnRegisterCLSIDInCategory(REFCLSID clsid, CATID catid)    
{
    ICatRegister* pcr = NULL ;    
	HRESULT hr = S_OK ;
    hr = CoCreateInstance(CLSID_StdComponentCategoriesMgr, 
            NULL, CLSCTX_INPROC_SERVER, IID_ICatRegister, (void**)&pcr);
    if (SUCCEEDED(hr))    
	{
       // Unregister this category as being "implemented" by the class.
       CATID rgcatid[1] ;       
	   rgcatid[0] = catid;
       hr = pcr->UnRegisterClassImplCategories(clsid, 1, rgcatid);    
	}
    if (pcr != NULL)        
		pcr->Release();        
	return hr;        
}





